﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Management;

namespace TweakAPI.Info
{
    /// <summary>
    /// Provides information about the graphics card(s).
    /// </summary>
    public struct GraphicsCard
    {
        /// <summary>
        /// Returns a <see cref="GraphicsCard"/>-Array containing information about all mounted graphics cards.
        /// </summary>
        /// <returns></returns>
        public static GraphicsCard[] EnumerateDevices()
        {
            ManagementObjectSearcher searcher = new ManagementObjectSearcher(@"root\CIMV2", "SELECT * FROM Win32_VideoController");
            List<GraphicsCard> infoList = new List<GraphicsCard>();
            foreach (ManagementObject queryObj in searcher.Get())
            {
                GraphicsCard info = new GraphicsCard();
                try { info.Name = queryObj["Name"].ToString(); }
                catch { }

                try { info.Memory = Math.Round((Convert.ToDecimal(queryObj["AdapterRAM"].ToString()) / 1024) / 1024, 1).ToString() + " MiB (" + Math.Round(((Convert.ToDecimal(queryObj["AdapterRAM"].ToString()) / 1024) / 1024) / 1024, 1).ToString() + " GiB)"; }
                catch { }

                try { info.Driver = queryObj["DriverVersion"].ToString(); }
                catch { }
                infoList.Add(info);
            }
            return infoList.ToArray();
        }

        /// <summary>
        /// The name.
        /// </summary>
        public string Name { get; set; }
        /// <summary>
        /// The amount of memory.
        /// </summary>
        public string Memory { get; set; }
        /// <summary>
        /// The driver caption.
        /// </summary>
        public string Driver { get; set; }
    }
}
