﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Management;

namespace TweakAPI.Info
{
    /// <summary>
    /// Provides information about the printer jobs.
    /// </summary>
    public struct PrinterJob
    {
        /// <summary>
        /// Returns a <see cref="PrinterJob"/>-Array containing information about the printer jobs.
        /// </summary>
        /// <returns></returns>
        public static PrinterJob[] EnumerateJobs()
        {
            List<PrinterJob> joblist = new List<PrinterJob>();
            ManagementObjectSearcher searcher = new ManagementObjectSearcher(@"root\CIMV2", "SELECT * FROM Win32_PrintJob");
            foreach (ManagementObject queryObj in searcher.Get())
            {
                PrinterJob printjob = new PrinterJob();
                printjob.Color = queryObj["Color"].ToString();
                printjob.DataType = queryObj["DataType"].ToString();
                printjob.Document = queryObj["Document"].ToString();
                printjob.Driver = queryObj["DriverName"].ToString();
                printjob.HostPrintQueue = queryObj["HostPrintQueue"].ToString();
                printjob.JobID = Convert.ToInt32(queryObj["JobId"].ToString());
                printjob.Notify = queryObj["Notify"].ToString();
                printjob.Owner = queryObj["Owner"].ToString();
                printjob.PagesPrinted = Convert.ToInt32(queryObj["PagesPrinted"].ToString());
                printjob.PaperLength = Convert.ToInt32(queryObj["PaperLength"].ToString());
                printjob.PaperSize = queryObj["PaperSize"].ToString();
                printjob.PaperWidth = Convert.ToInt32(queryObj["PaperWidth"].ToString());
                printjob.PrintProcessor = queryObj["PrintProcessor"].ToString();
                printjob.Priority = Convert.ToInt32(queryObj["Priority"].ToString());
                printjob.Status = queryObj["JobStatus"].ToString();
                printjob.TotalPages = Convert.ToInt32(queryObj["TotalPages"].ToString());
                joblist.Add(printjob);
            }
            return joblist.ToArray();
        }

        /// <summary>
        /// The color.
        /// </summary>
        public string Color { get; set; }
        /// <summary>
        /// The data type.
        /// </summary>
        public string DataType { get; set; }
        /// <summary>
        /// The document name.
        /// </summary>
        public string Document { get; set; }
        /// <summary>
        /// The driver.
        /// </summary>
        public string Driver { get; set; }
        /// <summary>
        /// The host print queue.
        /// </summary>
        public string HostPrintQueue { get; set; }
        /// <summary>
        /// The job ID.
        /// </summary>
        public int JobID { get; set; }
        /// <summary>
        /// The notify.
        /// </summary>
        public string Notify { get; set; }
        /// <summary>
        /// The owner.
        /// </summary>
        public string Owner { get; set; }
        /// <summary>
        /// The number of printed pages.
        /// </summary>
        public int PagesPrinted { get; set; }
        /// <summary>
        /// The paper length
        /// </summary>
        public int PaperLength { get; set; }
        /// <summary>
        /// The paper size.
        /// </summary>
        public string PaperSize { get; set; }
        /// <summary>
        /// The paper width.
        /// </summary>
        public int PaperWidth { get; set; }
        /// <summary>
        /// The print processor.
        /// </summary>
        public string PrintProcessor { get; set; }
        /// <summary>
        /// The priority.
        /// </summary>
        public int Priority { get; set; }
        /// <summary>
        /// The status.
        /// </summary>
        public string Status { get; set; }
        /// <summary>
        /// The total number of pages.
        /// </summary>
        public int TotalPages { get; set; }
    }
}
