﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Management;

namespace TweakAPI.Info
{
    /// <summary>
    /// Provides general information about the random access memory.
    /// </summary>
    public static class Ram
    {
        /// <summary>
        /// Returns the total amount of physical memory.
        /// </summary>
        /// <returns></returns>
        public static ulong TotalPhysicalMemory
        {
            get
            {
                Helper.MEMORYSTATUSEX mem = new Helper.MEMORYSTATUSEX();
                Win32Api.GlobalMemoryStatusEx(mem);
                ulong value = 0;
                value = (mem.ullTotalPhys / 1024uL) / 1024uL;      
                return value;
            }
        }

        /// <summary>
        /// Returns the available amount of physical memory.
        /// </summary>
        public static ulong AvailablePhysicalMemory
        {
            get
            {
                Helper.MEMORYSTATUSEX mem = new Helper.MEMORYSTATUSEX();
                Win32Api.GlobalMemoryStatusEx(mem);
                ulong value = 0;
                value = (mem.ullAvailPhys / 1024uL) / 1024uL;
                return value;
            }
        }

        /// <summary>
        /// Returns the total amount of virtual memory.
        /// </summary>
        /// <returns></returns>
        public static ulong TotalVirtualMemory
        {
            get
            {
                Helper.MEMORYSTATUSEX mem = new Helper.MEMORYSTATUSEX();
                Win32Api.GlobalMemoryStatusEx(mem);
                ulong value = 0;
                value = (mem.ullTotalVirtual / 1024uL) / 1024uL;
                return value;
            }
        }

        /// <summary>
        /// Returns the available amount of virtual memory.
        /// </summary>
        public static ulong AvailableVirtualMemory
        {
            get
            {
                Helper.MEMORYSTATUSEX mem = new Helper.MEMORYSTATUSEX();
                Win32Api.GlobalMemoryStatusEx(mem);
                ulong value = 0;
                value = (mem.ullAvailVirtual / 1024uL) / 1024uL;
                return value;
            }
        }

        /// <summary>
        /// Returns the amount of the currently used physical memory.
        /// </summary>
        /// <returns></returns>
        public static ulong UsedRAM
        {
            get
            {
                ulong allram = TotalPhysicalMemory;
                ulong availableram = AvailablePhysicalMemory;
                ulong useram = 0;
                useram = allram - availableram;
                        
                return useram;
            }
        }
    }
}
