﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Management;

namespace TweakAPI.Info
{
    /// <summary>
    /// Provides information about the mounted usb controllers.
    /// </summary>
    public struct UsbController
    {
        private static ManagementObjectSearcher searcher = new ManagementObjectSearcher(@"root\CIMV2", "SELECT * FROM Win32_USBController");

        /// <summary>
        /// Returns an <see cref="UsbController"/>-Array containing information about the mounted usb controllers.
        /// </summary>
        public static UsbController[] EnumerateControllers()
        {
            List<UsbController> usblist = new List<UsbController>();
            foreach (ManagementObject queryObj in searcher.Get())
            {
                UsbController usb = new UsbController();
                try { usb.Name = queryObj["Name"].ToString(); }
                catch { }

                try { usb.DeviceID = queryObj["DeviceID"].ToString(); }
                catch { }

                try { usb.Manufacturer = queryObj["Manufacturer"].ToString(); }
                catch { }

                try { usb.PowerManagementSupported = Convert.ToBoolean(queryObj["PowerManagementSupported"].ToString()); }
                catch { }

                try { usb.Status = queryObj["Status"].ToString(); }
                catch { }

                usblist.Add(usb);
            }
            return usblist.ToArray();
        }

        /// <summary>
        /// The name.
        /// </summary>
        public string Name { get; set; }
        /// <summary>
        /// The device ID.
        /// </summary>
        public string DeviceID { get; set; }
        /// <summary>
        /// The manufacturer.
        /// </summary>
        public string Manufacturer { get; set; }
        /// <summary>
        /// Information, whether power management is supported.
        /// </summary>
        public bool PowerManagementSupported { get; set; }
        /// <summary>
        /// The status.
        /// </summary>
        public string Status { get; set; }
    }
}
