﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Win32;

namespace TweakAPI.Tweak
{
    /// <summary>
    /// Provides possibilities to customize the context menu.
    /// </summary>
    public static class ContextMenu
    {

        /// <summary>
        /// Creates/removes the context menu entry 'Open command prompt here' (At the moment only in german).
        /// </summary>
        public static bool OpenCommandPromptHere
        {
            get
            {
                string ans = "";
                RegistryKey regKey = Registry.ClassesRoot.OpenSubKey("Directory\\shell\\cmd", RegistryKeyPermissionCheck.ReadSubTree);
                try
                {
                    ans = regKey.GetValue("Extended").ToString();
                    return false;
                }
                catch { return true; }
            }
            set
            {
                if (value)
                {
                    Registry.ClassesRoot.OpenSubKey("Directory\\shell\\cmd", RegistryKeyPermissionCheck.ReadWriteSubTree)
                        .SetValue("", "Kommandozeile hier öffnen", RegistryValueKind.String);
                    Registry.ClassesRoot.OpenSubKey("Directory\\shell\\cmd", RegistryKeyPermissionCheck.ReadWriteSubTree)
                        .DeleteValue("Extended");
                    Registry.ClassesRoot.OpenSubKey("Drive\\shell\\cmd", RegistryKeyPermissionCheck.ReadWriteSubTree)
                        .SetValue("", "Kommandozeile hier öffnen", RegistryValueKind.String);
                    Registry.ClassesRoot.OpenSubKey("Drive\\shell\\cmd", RegistryKeyPermissionCheck.ReadWriteSubTree)
                        .DeleteValue("Extended");
                }
                else
                {
                    Registry.ClassesRoot.OpenSubKey("Directory\\shell\\cmd", RegistryKeyPermissionCheck.ReadWriteSubTree)
                        .SetValue("Extended", "", RegistryValueKind.String);
                    Registry.ClassesRoot.OpenSubKey("Drive\\shell\\cmd", RegistryKeyPermissionCheck.ReadWriteSubTree)
                        .SetValue("Extended", "", RegistryValueKind.String);
                }
            }
        }

        /// <summary>
        /// Creates/removes the context menu entry 'Copy To'.
        /// </summary>
        public static bool CopyTo
        {
            get
            {
                RegistryKey regKey = Registry.ClassesRoot.OpenSubKey("AllFilesystemObjects\\shellex\\ContextMenuHandlers\\Copy to",
                    RegistryKeyPermissionCheck.ReadSubTree);

                if (regKey == null)
                { return false; }
                else
                { return true; }
            }
            set
            {
                if (value)
                {
                    RegistryKey regKey = Registry.ClassesRoot.OpenSubKey("AllFilesystemObjects\\shellex\\ContextMenuHandlers",
                        RegistryKeyPermissionCheck.ReadWriteSubTree).CreateSubKey("Copy to",
                        RegistryKeyPermissionCheck.ReadWriteSubTree);
                    
                    regKey.SetValue("", "{C2FBB630-2971-11D1-A18C-00C04FD75D13}", RegistryValueKind.String);
                }
                else
                {
                    Registry.ClassesRoot.OpenSubKey("AllFilesystemObjects\\shellex\\ContextMenuHandlers",
                        RegistryKeyPermissionCheck.ReadWriteSubTree).DeleteSubKeyTree("Copy to");
                }
            }
        }

        /// <summary>
        /// Creates/removes the context menu entry 'Move To'.
        /// </summary>
        public static bool MoveTo
        {
            get
            {
                RegistryKey regKey = Registry.ClassesRoot.OpenSubKey("AllFilesystemObjects\\shellex\\ContextMenuHandlers\\Move to", 
                    RegistryKeyPermissionCheck.ReadSubTree);
                if (regKey == null)
                { return false; }
                else
                { return true; }
            }
            set
            {
                if (value)
                {
                    RegistryKey regKey = Registry.ClassesRoot.OpenSubKey("AllFilesystemObjects\\shellex\\ContextMenuHandlers", 
                        RegistryKeyPermissionCheck.ReadWriteSubTree).CreateSubKey("Move to", 
                        RegistryKeyPermissionCheck.ReadWriteSubTree);

                    regKey.SetValue("", "{C2FBB631-2971-11D1-A18C-00C04FD75D13}", RegistryValueKind.String);
                }
                else
                {
                    Registry.ClassesRoot.OpenSubKey("AllFilesystemObjects\\shellex\\ContextMenuHandlers", 
                        RegistryKeyPermissionCheck.ReadWriteSubTree).DeleteSubKeyTree("Move to");
                }
            }
        }

        /// <summary>
        /// Creates/removes the context menu entry 'Send To'.
        /// </summary>
        public static bool SendTo
        {
            get
            {
                RegistryKey regKey = Registry.ClassesRoot.OpenSubKey("AllFilesystemObjects\\shellex\\ContextMenuHandlers\\SendTo", 
                    RegistryKeyPermissionCheck.ReadSubTree);
                if (regKey.GetValue("").ToString() == "{7BA4C740-9E81-11CF-99D3-00AA004AE837}")
                { return true; }
                else
                { return false; }
            }
            set
            {
                RegistryKey RegKey = Registry.ClassesRoot.OpenSubKey("AllFilesystemObjects\\shellex\\ContextMenuHandlers\\SendTo", 
                    RegistryKeyPermissionCheck.ReadWriteSubTree);
                if (value)
                { RegKey.SetValue("", "{7BA4C740-9E81-11CF-99D3-00AA004AE837}", RegistryValueKind.String); }
                else
                { RegKey.SetValue("", "", RegistryValueKind.String); }
            }
        }

        /// <summary>
        /// Creates/removes the context menu entry 'Copy to clipboard' (At the moment only in german).
        /// </summary>
        public static bool CopyToClipboard
        {
            get
            {
                RegistryKey regKey = Registry.ClassesRoot.OpenSubKey("txtfile\\shell\\copytoclip", 
                    RegistryKeyPermissionCheck.ReadSubTree);

                if (regKey == null)
                { return false; }
                else
                { return true; }
            }
            set
            {
                if (value)
                {
                    RegistryKey regKey = Registry.ClassesRoot.OpenSubKey("txtfile\\shell\\copytoclip", true);
                    if (regKey == null)
                    {
                        regKey = Registry.ClassesRoot.OpenSubKey("txtfile\\shell", RegistryKeyPermissionCheck.ReadWriteSubTree)
                            .CreateSubKey("copytoclip", RegistryKeyPermissionCheck.ReadWriteSubTree);
                        regKey.SetValue("", "Inhalt in Zwischenablage kopieren", RegistryValueKind.String);
                        regKey = regKey.CreateSubKey("command", RegistryKeyPermissionCheck.ReadWriteSubTree);                        
                    }
                    else
                    { regKey = regKey.OpenSubKey("command", RegistryKeyPermissionCheck.ReadWriteSubTree); }
                                        
                    regKey.SetValue("", "cmd /c clip < \"%1\""); 
                }
                else
                {
                    Registry.ClassesRoot.OpenSubKey(@"txtfile\shell", RegistryKeyPermissionCheck.ReadWriteSubTree)
                        .DeleteSubKeyTree("copytoclip");
                }
            }
        }

        /// <summary>
        /// Creates/removes the context menu entry 'Map Network Drive'.
        /// </summary>
        public static bool MapNetworkDrive
        {
            get
            {
                int ans = 0;
                RegistryKey regKey = Registry.CurrentUser.OpenSubKey("Software\\Microsoft\\Windows\\CurrentVersion" + 
                    "\\Policies\\Explorer", RegistryKeyPermissionCheck.ReadSubTree);
                if (regKey == null)
                { return true; }

                try { ans = int.Parse(regKey.GetValue("NoNetConnectDisconnect").ToString()); }
                catch { return true; }

                if (ans == 0)
                { return true; }
                else
                { return false; }
            }
            set
            {
                RegistryKey regKey = Registry.CurrentUser.OpenSubKey("Software\\Microsoft\\Windows\\CurrentVersion" + 
                    "\\Policies\\Explorer", RegistryKeyPermissionCheck.ReadWriteSubTree);
                if (regKey == null)
                {
                    regKey = Registry.CurrentUser.OpenSubKey("Software\\Microsoft\\Windows\\CurrentVersion\\Policies", 
                        RegistryKeyPermissionCheck.ReadWriteSubTree).CreateSubKey("Explorer", RegistryKeyPermissionCheck.ReadWriteSubTree);
                }

                if (value)
                { regKey.SetValue("NoNetConnectDisconnect", 0, RegistryValueKind.DWord); }
                else
                { regKey.SetValue("NoNetConnectDisconnect", 1, RegistryValueKind.DWord); }
            }
        }

        /// <summary>
        /// Creates/removes the context menu entry 'Defrag' (At the moment only in german).
        /// </summary>
        public static bool Defragment
        {
            get
            {
                RegistryKey regKey = Registry.ClassesRoot.OpenSubKey("Drive\\shell\\runas", RegistryKeyPermissionCheck.ReadSubTree);
                if (regKey == null)
                { return false; }
                else
                { return true; }
            }
            set
            {
                if (value)
                {
                    RegistryKey regKey = Registry.ClassesRoot.OpenSubKey("Drive\\shell\\runas", 
                        RegistryKeyPermissionCheck.ReadWriteSubTree);
                    if (regKey == null)
                    {
                        regKey = Registry.ClassesRoot.OpenSubKey("Drive\\shell", RegistryKeyPermissionCheck.ReadWriteSubTree)
                            .CreateSubKey("runas", RegistryKeyPermissionCheck.ReadWriteSubTree);

                        regKey.SetValue("", "Defragmentieren", RegistryValueKind.String);
                        regKey.SetValue("icon", "defrag.exe", RegistryValueKind.String);
                        regKey = regKey.CreateSubKey("command", RegistryKeyPermissionCheck.ReadWriteSubTree);
                    }
                    else
                    { regKey = regKey.OpenSubKey("command", RegistryKeyPermissionCheck.ReadWriteSubTree); }
                    
                    regKey.SetValue("", "dfrgui.exe", RegistryValueKind.String);
                }
                else
                {
                    Registry.ClassesRoot.OpenSubKey("Drive\\shell", RegistryKeyPermissionCheck.ReadWriteSubTree)
                        .DeleteSubKeyTree("runas");
                }
            }
        }

        /// <summary>
        /// Creates/removes the context menu entry 'Control Panel' (At the moment only in german).
        /// </summary>
        public static bool ControlPanel
        {
            get
            {
                bool ret = false;
                RegistryKey RegKey = Registry.ClassesRoot.OpenSubKey("Directory\\Background\\shell\\ControlPanel", true);
                if (RegKey != null)
                { ret = true; }
                return ret;
            }
            set
            {
                if (value)
                {
                    RegistryKey regKey = Registry.ClassesRoot.OpenSubKey("Directory\\Background\\shell\\ControlPanel", 
                        RegistryKeyPermissionCheck.ReadWriteSubTree);
                    if (regKey == null)
                    {
                        regKey = Registry.ClassesRoot.OpenSubKey("Directory\\Background\\shell", 
                            RegistryKeyPermissionCheck.ReadWriteSubTree).CreateSubKey("ControlPanel", 
                            RegistryKeyPermissionCheck.ReadWriteSubTree);
                        regKey.SetValue("", "Systemsteuerung", RegistryValueKind.String);
                        regKey = regKey.CreateSubKey("command", RegistryKeyPermissionCheck.ReadWriteSubTree);
                    }
                    else
                    { regKey = regKey.OpenSubKey("command", RegistryKeyPermissionCheck.ReadWriteSubTree); }
                    regKey.SetValue("", "rundll32.exe shell32.dll,Control_RunDLL", RegistryValueKind.String);
                }
                else
                {
                    Registry.ClassesRoot.OpenSubKey("Directory\\Background\\shell", RegistryKeyPermissionCheck.ReadWriteSubTree)
                        .DeleteSubKeyTree("ControlPanel");
                }
            }
        }

        /// <summary>
        /// Creates/removes the context menu entries 'Encrypt' and 'Decrypt'.
        /// </summary>
        public static bool Encryption
        {
            get
            {
                int ans = 0;
                RegistryKey regKey = Registry.CurrentUser.OpenSubKey("Software\\Microsoft\\Windows\\CurrentVersion" + 
                    "\\Explorer\\Advanced", RegistryKeyPermissionCheck.ReadSubTree);
                if (regKey == null)
                { return false; }

                try { ans = int.Parse(regKey.GetValue("EncryptionContextMenu").ToString()); }
                catch { return false; }
                if (ans == 1)
                { return true; }
                else
                { return false; }
            }
            set
            {
                RegistryKey regKey = Registry.CurrentUser.OpenSubKey("Software\\Microsoft\\Windows\\CurrentVersion" + 
                    "\\Explorer\\Advanced", RegistryKeyPermissionCheck.ReadWriteSubTree);
                if (value)
                { regKey.SetValue("EncryptionContextMenu", 1, RegistryValueKind.DWord); }
                else
                { regKey.SetValue("EncryptionContextMenu", 0, RegistryValueKind.DWord); }
            }
        }

        /// <summary>
        /// Creates/removes the context menu entry 'Take ownership' (At the moment only in german).
        /// </summary>
        public static bool TakeOwnership
        {
            get
            {
                RegistryKey regKey = Registry.ClassesRoot.OpenSubKey("*\\shell\\runas", RegistryKeyPermissionCheck.ReadSubTree);
                RegistryKey regKey2 = Registry.ClassesRoot.OpenSubKey("Directory\\shell\\runas", 
                    RegistryKeyPermissionCheck.ReadSubTree);

                if (regKey == null && regKey2 == null)
                { return false; }
                else
                { return true; }
            }
            set
            {
                if (value)
                {
                    RegistryKey regKey = Registry.ClassesRoot.OpenSubKey("*\\shell\\runas", 
                        RegistryKeyPermissionCheck.ReadWriteSubTree);
                    RegistryKey regKey2 = Registry.ClassesRoot.OpenSubKey("Directory\\shell\\runas",
                        RegistryKeyPermissionCheck.ReadSubTree);

                    if (regKey == null)
                    {
                        regKey = Registry.ClassesRoot.OpenSubKey("*\\shell", RegistryKeyPermissionCheck.ReadWriteSubTree)
                            .CreateSubKey("runas", RegistryKeyPermissionCheck.ReadWriteSubTree);
                        regKey.SetValue("", "In Besitz nehmen");
                        regKey.SetValue("NoWorkingDirectory", "", RegistryValueKind.String);
                        regKey = regKey.CreateSubKey("command");

                        regKey2 = Registry.ClassesRoot.OpenSubKey("Directory\\shell", RegistryKeyPermissionCheck.ReadWriteSubTree)
                            .CreateSubKey("runas", RegistryKeyPermissionCheck.ReadWriteSubTree);
                        regKey2.SetValue("", "In Besitz nehmen");
                        regKey2.SetValue("NoWorkingDirectory", "", RegistryValueKind.String);
                        regKey2 = regKey2.CreateSubKey("command");
                    }
                    else
                    {
                        regKey = regKey.OpenSubKey("command", RegistryKeyPermissionCheck.ReadWriteSubTree);
                        regKey2 = regKey2.OpenSubKey("command", RegistryKeyPermissionCheck.ReadWriteSubTree);
                    }
                                        
                    regKey.SetValue("", "cmd.exe /c takeown /f \"%1\" && icacls \"%1\" /grant administrators:F", 
                        RegistryValueKind.String);
                    regKey.SetValue("IsolatedCommand", "cmd.exe /c takeown /f \"%1\" && icacls \"%1\" /grant administrators:F", 
                        RegistryValueKind.String);
                                        
                    regKey2.SetValue("", "cmd.exe /c takeown /f \"%1\" /r /d y && icacls \"%1\" /grant administrators:F /t", 
                        RegistryValueKind.String);
                    regKey2.SetValue("IsolatedCommand", "cmd.exe /c takeown /f \"%1\" /r /d y && icacls \"%1\" /grant " + 
                        "administrators:F /t", RegistryValueKind.String);
                }
                else
                {
                    Registry.ClassesRoot.OpenSubKey("*\\shell", RegistryKeyPermissionCheck.ReadWriteSubTree).DeleteSubKeyTree("runas");
                    Registry.ClassesRoot.OpenSubKey("Directory\\shell", RegistryKeyPermissionCheck.ReadWriteSubTree)
                        .DeleteSubKeyTree("runas");
                }
            }
        }
    }
}
