﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Runtime.InteropServices;
using System.Drawing;
using System.Net;

namespace TweakAPI.Tweak
{
    /// <summary>
    /// Provides the usage of the Windows credential dialog.
    /// </summary>
    public class CredentialDialog
    {
        [DllImport("ole32.dll")]
        private static extern void CoTaskMemFree(IntPtr ptr);

        [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Auto)]
        private struct CREDUI_INFO
        {
            public int cbSize;
            public IntPtr hwndParent;
            public string pszMessageText;
            public string pszCaptionText;
            public IntPtr hbmBanner;
        }

        [DllImport("credui.dll", CharSet = CharSet.Auto)]
        private static extern bool CredUnPackAuthenticationBuffer(int dwFlags, IntPtr pAuthBuffer, uint cbAuthBuffer, StringBuilder pszUserName, ref int pcchMaxUserName, StringBuilder pszDomainName,
            ref int pcchMaxDomainame, StringBuilder pszPassword, ref int pcchMaxPassword);

        [DllImport("credui.dll", CharSet = CharSet.Auto)]
        private static extern int CredUIPromptForWindowsCredentials(ref CREDUI_INFO notUsedHere, int authError, ref uint authPackage, IntPtr InAuthBuffer, uint InAuthBufferSize, ref IntPtr refOutAuthBuffer,
            ref uint refOutAuthBufferSize, ref bool fSave, int flags);

        [Flags()]
        private enum CredUIWinFlags
        {
            CredUIWin_Generic = 0x1,
            CredUIWin_Checkbox = 0x2,
            CredUIWin_AuthPackage_Only = 0x10,
            CredUIWin_InCred_Only = 0x20,
            CredUIWin_Enumerate_Admins = 0x100,
            CredUIWin_Enumerate_CurrentUser = 0x200,
            CredUIWin_Secure_Prompt = 0x1000,
            CredUIWin_Pack_32_Wow = 0x10000000,
        }

        /// <summary>
        /// Shows the credential dialog.
        /// </summary>
        /// <param name="caption"></param>
        /// <param name="message"></param>
        /// <param name="netCred"></param>
        /// <param name="image"></param>
        public static void Show(string caption, string message, ref NetworkCredential netCred, Bitmap image)
        {
            CREDUI_INFO credInf = new CREDUI_INFO();
            credInf.pszCaptionText = caption;
            credInf.pszMessageText = message;

            if (image != null)
            {
                credInf.hbmBanner = image.GetHbitmap();
            }
            credInf.cbSize = Marshal.SizeOf(credInf);
            uint authPackage = 0;
            IntPtr outCredBuffer = new IntPtr();
            uint outCredSize = 0;
            bool save = false;

            int result = CredUIPromptForWindowsCredentials(ref credInf, 0, ref authPackage, IntPtr.Zero, 0, ref outCredBuffer,
                ref outCredSize, ref save, Convert.ToInt32(CredUIWinFlags.CredUIWin_Generic));

            StringBuilder usernameBuf = new StringBuilder(100);
            StringBuilder passwordBuf = new StringBuilder(100);
            StringBuilder domainBuf = new StringBuilder(100);

            int maxUserName = 100;
            int maxDomain = 100;
            int maxPassword = 100;
            if (result == 0)
            {
                if (CredUnPackAuthenticationBuffer(0, outCredBuffer, outCredSize, usernameBuf, ref maxUserName, domainBuf,
                    ref maxDomain, passwordBuf, ref maxPassword))
                {
                    CoTaskMemFree(outCredBuffer);
                    netCred = new NetworkCredential(usernameBuf.ToString(), passwordBuf.ToString(), domainBuf.ToString());
                    return;
                }
            }

            netCred = null;
        }
    }
}
