﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Runtime.InteropServices;

namespace TweakAPI
{
    /// <summary>
    /// Provides Windows API methods.
    /// </summary>
    public static class Win32Api
    {
        #region Graphic/Monitor

        [StructLayout(LayoutKind.Sequential)]
        public struct POINTL
        {
            public int x;
            public int y;
        }

        [StructLayout(LayoutKind.Sequential)]
        public struct DEVMODE
        {
            [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 32)]
            public string dmDeviceName;
            public short dmSpecVersion;
            public short dmDriverVersion;
            public short dmSize;
            public short dmDriverExtra;
            public int dmFields;

            public POINTL dmPosition;
            public int dmDisplayOrientation;
            public int DisplayFixedOutput;

            public short dmColor;
            public short dmDuplex;
            public short dmYResolution;
            public short dmTTOption;
            public short dmCollate;
            [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 32)]
            public string dmFormName;
            public short dmLogPixels;
            public int dmBitsPerPel;
            public int dmPelsWidth;
            public int dmPelsHeight;

            public int dmDisplayFlags;
            public int dmDisplayFrequency;

            public int dmICMMethod;
            public int dmICMIntent;
            public int dmMediaType;
            public int dmDitherType;
            public int dmReserved1;
            public int dmReserved2;

            public int dmPanningWidth;
            public int dmPanningHeight;
        }

        [DllImport("user32.dll")]
        public static extern bool EnumDisplaySettings(string deviceName, int modeNum, ref DEVMODE devMode);

        [DllImport("user32.dll")]
        public static extern DISP_CHANGE ChangeDisplaySettingsEx(string lpszDeviceName, ref DEVMODE lpDevMode, IntPtr hwnd, uint dwflags, IntPtr lParam);

        public enum DISP_CHANGE
        {
            Successful = 0,
            Restart = 1,
            Failed = -1,
            BadMode = -2,
            NotUpdated = -3,
            BadFlags = -4,
            BadParam = -5,
            BadDualView = -6,
        }

        public const int ENUM_CURRENT_SETTINGS = -1;
        public const int CDS_UPDATEREGISTRY = 0x1;
        public const int CDS_TEST = 0x2;

        public const int DM_DISPLAYORIENTATION = 0x80;
        public const int DM_PELSWIDTH = 0x80000;
        public const int DM_PELSHEIGHT = 0x100000;

        public static void SwapExtents(ref int pelHeight, ref int pelWidth)
        {
            pelHeight = pelHeight ^ pelWidth;
            pelWidth = pelWidth ^ pelHeight;
            pelHeight = pelHeight ^ pelWidth;
        }

        [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Ansi)]
        public struct RAMP
        {
            [MarshalAs(UnmanagedType.ByValArray, SizeConst = 256)]
            public UInt16[] Red;
            [MarshalAs(UnmanagedType.ByValArray, SizeConst = 256)]
            public UInt16[] Green;
            [MarshalAs(UnmanagedType.ByValArray, SizeConst = 256)]
            public UInt16[] Blue;
        }

        [DllImport("gdi32.dll")]
        public static extern bool SetDeviceGammaRamp(IntPtr hDC, ref RAMP lpRamp);

        [DllImport("user32.dll")]
        public static extern IntPtr GetDC(IntPtr hWnd);

        [DllImport("gdi32.dll")]
        public static extern bool GetDeviceGammaRamp(Int32 hdc, ref RAMP lpv);

        public static T InlineAssignHelper<T>(ref T target, T value)
        {
            target = value;
            return value;
        }

        [DllImport("kernel32.dll")]
        public static extern IntPtr CreateFile(string lpFileName, uint dwDesiredAccess, uint dwShareMode,
            IntPtr lpSecurityAttributes, Int32 dwCreationDisposition, Int32 dwFlagsAndAttributes,
            IntPtr hTemplateFile);

        public const uint GENERIC_READ = 0x80000000;
        public const int SHARE_ALL = 0x7;
        public const int OPEN_EXISTING = 0x3;

        public const int IOCTL_VIDEO_SET_DISPLAY_BRIGHTNESS = 0x23049C;

        public struct BRIGHTNESS
        {
            public byte ucDisplayPolicy;
            public byte ucACBrightness;
            public byte ucDCBrightness;
        }

        [DllImport("kernel32.dll")]
        public static extern bool DeviceIoControl(IntPtr hDevice, Int32 dwIoControlCode, ref BRIGHTNESS lpInBuffer,
            Int32 nInBufferSize, IntPtr lpOutBuffer,
            Int32 nOutBufferSize, ref Int32 lpBytesReturned, IntPtr lpOverlapped);

        [DllImport("kernel32.dll")]
        public static extern bool CloseHandle(IntPtr hObject);

        [DllImport("user32.dll", EntryPoint = "SendMessageA")]
        public static extern void SendMessage(IntPtr hWnd, Int32 uMsg, Int32 wParam, Int32 lParam);

        public enum Params
        {
            SC_MONITORPOWER = 0xF170,
            WM_SYSCOMMAND = 0x112,
            TURN_MONITOR_ON = -1,
            TURN_MONITOR_OFF = 2,
        }

        #endregion

        #region Ram

        [return: MarshalAs(UnmanagedType.Bool)]
        [DllImport("kernel32.dll", CharSet = CharSet.Auto, SetLastError = true)]
        public static extern bool GlobalMemoryStatusEx([In, Out] Helper.MEMORYSTATUSEX lpBuffer);

        #endregion

        #region Desktop

        public const int NOCONFIRMATION = 1;
        public const int NOPROGRESS = 2;
        public const short NOSOUND = 4;

        [DllImport("shell32.dll")]
        public static extern int SHEmptyRecycleBin(IntPtr hWnd, string pszRootPath, uint dwFlags);

        [DllImport("shell32.dll")]
        public static extern int SHUpdateRecycleBinIcon();

        [DllImport("shell32.dll")]
        public static extern void SHChangeNotify(HChangeNotifyEventID wEventID, HChangeNotifyFlags uFlags, IntPtr dwItem1, IntPtr dwItem2);

        [Flags]
        public enum HChangeNotifyEventID
        {
            SHCNE_ALLEVENTS = 0x7FFFFFFF,
            SHCNE_ASSOCCHANGED = 0x8000000,
            SHCNE_ATTRIBUTES = 0x800,
            SHCNE_CREATE = 0x2,
            SHCNE_DELETE = 0x4,
            SHCNE_DRIVEADD = 0x100,
            SHCNE_DRIVEADDGUI = 0x10000,
            SHCNE_DRIVEREMOVED = 0x80,
            SHCNE_EXTENDED_EVENT = 0x4000000,
            SHCNE_FREESPACE = 0x40000,
            SHCNE_MEDIAINSERTED = 0x20,
            SHCNE_MEDIAREMOVED = 0x40,
            SHCNE_MKDIR = 0x8,
            SHCNE_NETSHARE = 0x200,
            SHCNE_NETUNSHARE = 0x400,
            SHCNE_RENAMEFOLDER = 0x20000,
            SHCNE_RENAMEITEM = 0x1,
            SHCNE_RMDIR = 0x10,
            SHCNE_SERVERDISCONNECT = 0x4000,
            SHCNE_UPDATEDIR = 0x1000,
            SHCNE_UPDATEIMAGE = 0x8000,
        }

        [Flags]
        public enum HChangeNotifyFlags
        {
            SHCNF_DWORD = 0x3,
            SHCNF_IDLIST = 0x0,
            SHCNF_PATHA = 0x1,
            SHCNF_PATHW = 0x5,
            SHCNF_PRINTERA = 0x2,
            SHCNF_PRINTERW = 0x6,
            SHCNF_FLUSH = 0x1000,
            SHCNF_FLUSHNOWAIT = 0x2000,
        }

        [DllImport("user32.dll")]
        public static extern int SystemParametersInfo(int uAction, int uParam, string pncMetrics, int fuWinIni);

        public const Int32 SPI_SETDESKWALLPAPER = 20;
        public const Int32 SPIF_UPDATEINIFILE = 0x1;
        public const Int32 SPIF_SENDWININICHANGE = 0x2;

        [DllImport("user32.dll")]
        public static extern int GetWindowText(IntPtr hWnd, StringBuilder text, int count);

        [DllImport("user32.dll", CharSet = CharSet.Auto)]
        public static extern bool EnumThreadWindows(int threadId, EnumThreadProc pfnEnum, IntPtr lParam);

        [DllImport("user32.dll", SetLastError = true)]
        public static extern IntPtr FindWindow(string lpClassName, string lpWindowName);

        [DllImport("user32.dll", SetLastError = true)]
        public static extern IntPtr FindWindowEx(IntPtr parentHandle, IntPtr childAfter, string className, string windowTitle);

        [DllImport("user32.dll")]
        public static extern IntPtr FindWindowEx(IntPtr parentHwnd, IntPtr childAfterHwnd, IntPtr className, string windowText);

        [DllImport("user32.dll")]
        public static extern int ShowWindow(IntPtr hwnd, int nCmdShow);

        [DllImport("user32.dll")]
        public static extern uint GetWindowThreadProcessId(IntPtr hwnd, ref int lpdwProcessId);

        public const int SW_HIDE = 0;
        public const int SW_SHOW = 5;

        public delegate bool EnumThreadProc(IntPtr hwnd, IntPtr lParam);

        #endregion

        #region Devices

        [DllImport("winmm.dll")]
        public static extern long mciExecute(string IpstrCommand);

        #endregion
    }
}
